/* SPDX-License-Identifier: GPL-2.0 */
#ifndef _ASM_GENERIC_BITOPS_CONST_PARITY_H_
#define _ASM_GENERIC_BITOPS_CONST_PARITY_H_

/*
 * Compile time versions of __arch_parityN()
 */
#define __const_parity4(w) ((0x6996 >> ((w) & 0xf)) & 1)
#define __const_parity8(w) (__const_parity4((w) ^ (w) >> 4))
#define __const_parity16(w) (__const_parity8((w) ^ (w) >> 8))
#define __const_parity32(w) (__const_parity16((w) ^ (w) >> 16))
#define __const_parity64(w) (__const_parity32((w) ^ (w) >> 32))

/*
 * Generic interface.
 */
#define parity8(w)  (__builtin_constant_p(w) ? __const_parity8(w)  : __arch_parity8(w))
#define parity16(w) (__builtin_constant_p(w) ? __const_parity16(w) : __arch_parity16(w))
#define parity32(w) (__builtin_constant_p(w) ? __const_parity32(w) : __arch_parity32(w))
#define parity64(w) (__builtin_constant_p(w) ? __const_parity64(w) : __arch_parity64(w))

/*
 * Interface for known constant arguments
 */
#define PARITY8(w)  (BUILD_BUG_ON_ZERO(!__builtin_constant_p(w)) + __const_parity8(w))
#define PARITY16(w) (BUILD_BUG_ON_ZERO(!__builtin_constant_p(w)) + __const_parity16(w))
#define PARITY32(w) (BUILD_BUG_ON_ZERO(!__builtin_constant_p(w)) + __const_parity32(w))
#define PARITY64(w) (BUILD_BUG_ON_ZERO(!__builtin_constant_p(w)) + __const_parity64(w))

/*
 * Type invariant interface to the compile time constant parity functions.
 */
#define PARITY(w)   PARITY64((u64)w)

#endif /* _ASM_GENERIC_BITOPS_CONST_PARITY_H_ */
